/**
 * Diese Klasse repraesentiert einen Bruch.
 * @author Peter Schneider-Kamp
 * @version 25.11.2003, 12:24
 */
public class Element {

    /**
     * Der Bruch wird als Zaehler und Nenner gespeichert.
     */
    private int z, n;

    /**
     * Diese Methode setzt den Wert der Klasse auf
     * das Nullelement.
     */
    public void setZero() {
        z = 0;
        n = 1;
    }

    /**
     * Diese Methode berechnet das Produkt von zwei Elementen.
     * Dabei wird ein neues Element erzeugt, d.h., die bergebenen
     * Elemente werden nicht gendert.
     * @param b Der zweite Multiplikand des Produktes.
     * @return Das Produkt der beiden Elemente.
     */
    public Element multiply(Element other) {
        Element ergebnis = new Element();
        ergebnis.z = z * other.z;
        ergebnis.n = n * other.n;
        ergebnis.cancel();
        return ergebnis;
    }

    /**
     * Diese Methode berechnet die Summe von zwei Elementen.
     * Dabei wird ein neues Element erzeugt, d.h., die bergebenen
     * Elemente werden nicht gendert.
     * @param b Der zweite Summand des Summe.
     * @return Die Summe der beiden Elemente.
     */
    public Element add(Element other) {
        Element ergebnis = new Element();
        ergebnis.z = z * other.n +
                     other.z * n;
        ergebnis.n = n * other.n;
        ergebnis.cancel();
        return ergebnis;
    }

    /**
     * Diese Methode liest ein Element ein.
     * @return Das eingelesene Element.
     */
    public static Element read() {
        Element neu = new Element();
        neu.z = IO.readInt();
        neu.n = IO.readInt();
        neu.cancel();
        return neu;
    }

    private void cancel() {
        int a = z;
        int b = n;
        while (b > 0) {
            int r = a % b;
            a = b;
            b = r;
        }
        z /= a;
        n /= a;
    }

    /**
     * Diese Methode erstellt eine Stringdarstellung dieses Elementes.
     * @return Eine Stringdarstellung dieses Elementes.
     */
    public String toString() {
        return "("+z+"/"+n+")";
    }

}
