/**
 * Diese Klasse enthaelt statische Funktionen
 * zum Umgang mit Matrizen.
 * @author Peter Schneider-Kamp
 * @version 25.11.2003, 11:26
 */
public class MatrizenRechnung {

    /**
     * Diese Methode berechnet die transponierte einer Matrix.
     * Dabei wird eine neue Matrix erzeugt, d.h., die bergebene
     * Matrix wird nicht gendert.
     * @param original Die Matrix, die transponiert werden soll.
     * @return Die transponierte Matrix.
     */
    public static Matrix transpose(Matrix original) {
        Matrix kopie = new Matrix();
        int hoehe = original.getHeight();
        int breite = original.getWidth();
        kopie.setDimension(hoehe, breite);
        for (int i = 0; i < hoehe; i++) {
            for (int j = 0; j < breite; j++) {
                kopie.setElement(i, j, original.getElement(j, i));
            }
        }
        return kopie;
    }

    /**
     * Diese Methode berechnet das Matrixprodukt von zwei Matrizen.
     * Dabei wird eine neue Matrix erzeugt, d.h., die bergebenen
     * Matrizen werden nicht gendert.
     * @param a Der erste Multiplikand des Produktes.
     * @param b Der zweite Multiplikand des Produktes.
     * @return Das Matrixprodukt der beiden Matrizen.
     */
    public static Matrix multiply(Matrix a, Matrix b) {
        Matrix produkt = new Matrix();
        produkt.setDimension(b.getWidth(), a.getHeight());
        for (int i = 0; i < a.getHeight(); i++) {
            for (int j = 0; j < b.getWidth(); j++) {
                Element w = new Element();
                w.setZero();
                for (int k = 0; k < a.getWidth(); k++) {
                    Element aki = a.getElement(k, i);
                    Element bjk = b.getElement(j, k);
                    w = w.add(aki.multiply(bjk));
                }
                produkt.setElement(j, i, w);
            }
        }
        return produkt;
    }

    /**
     * Diese Methode berechnet die Summe zweier Matrizen.
     * Dabei wird eine neue Matrix erzeugt, d.h., die bergebenen
     * Matrizen werden nicht gendert.
     * @param a Der erste Summand der Summe.
     * @param b Der zweite Summand des Summe.
     * @return Die Summe der beiden Matrizen.
     */
    public static Matrix add(Matrix a, Matrix b) {
        Matrix sum = new Matrix();
        sum.setDimension(a.getWidth(), a.getHeight());
        for (int i = 0; i < a.getWidth(); i++) {
            for (int j = 0; j < b.getHeight(); j++) {
                Element aij = a.getElement(i, j);
                Element bij = b.getElement(i, j);
                Element w = aij.add(bij);
                sum.setElement(i, j, w);
            }
        }
        return sum;
    }

    /**
     * Diese Methode liest eine Matrix einer gegebenen Breite und
     * Hoehe ein.
     * @param breite Die Breite der einzulesenden Matrix.
     * @param hoehe Die Hoehe der einzulesenden Matrix.
     * @return Die eingelesene Matrix.
     */
    public static Matrix read(int breite, int hoehe) {
        Matrix a = new Matrix();
        a.setDimension(breite, hoehe);
        for (int j = 0; j < hoehe; j++) {
            for (int i = 0; i < breite; i++) {
                a.setElement(i, j, Element.read());
            }
        }
        return a;
    }

}
